<?php

/**
 * @file
 * ContextIOMessageParser related classes.
 */

/**
 * Parse Context.IO specific email message data.
 */
class ContextIOMessageParser extends FeedsParser {

  /**
   * Implements FeedsParser::parse().
   */
  public function parse(FeedsSource $source, FeedsFetcherResult $fetcher_result) {
    $fetcher = $source->importer()->fetcher;
    // Check fetcher's type.
    if ((get_class($fetcher) == 'ContextIOMessageFetcher') && !empty($fetcher_result->getMessage())) {
      $message = $fetcher_result->getMessage();
      // Replace subject if there is a given regular expression.
      if (!empty($message['subject']) && !empty($this->config['subject_replace']['pattern'])) {
        $pattern = $this->config['subject_replace']['pattern'];
        $replacement = $this->config['subject_replace']['replacement'];
        $message['subject'] = preg_replace('/' . $pattern . '/', $replacement, $message['subject']);
      }
      // Result defaults.
      $result = array(
        'id' => $message['message_id'],
        'date' => $message['date'],
        'subject' => empty($message['subject']) ? '' : $message['subject'],
        'from_email' => empty($message['addresses']['from']['email']) ? '' : $message['addresses']['from']['email'],
        'from_name' => empty($message['addresses']['from']['name']) ? '' : $message['addresses']['from']['name'],
        'to_emails' => '',
        'flags' => '',
        'body_plain' => '',
        'body_html' => '',
      );

      // Make a list from "To" email addresses.
      if (!empty($message['addresses']['to'])) {
        $to_count = count($message['addresses']['to']) - 1;
        foreach ($message['addresses']['to'] as $index => $to) {
          $result['to_emails'] .= $to['email'];
          if ($index < $to_count) {
            $result['to_emails'] .= ', ';
          }
        }
      }

      // Make a list from flags.
      if (!empty($message['flags'])) {
        $result['flags'] = implode(', ', $message['flags']);
      }

      // Check message bodies. For now plain text and HTML type supported.
      if (!empty($message['body'])) {
        foreach ($message['body'] as $body) {
          switch ($body['type']) {
            case 'text/plain':
              $result['body_plain'] = $body['content'];
              break;

            case 'text/html':
              $result['body_html'] = $body['content'];
              break;
          }
        }
      }

      return new FeedsParserResult(array($result));
    }
    return new FeedsParserResult(array());
  }

  /**
   * Return mapping sources.
   */
  public function getMappingSources() {
    $sources = array();
    $fetcher = feeds_importer($this->id)->fetcher;
    // Check fetcher' type.
    if (get_class($fetcher) == 'ContextIOMessageFetcher') {
      // Default sources.
      $sources += array(
        'id' => array(
          'name' => t('Id'),
          'description' => t('The unique and persistent id assigned by Context.IO to the message.'),
        ),
        'date' => array(
          'name' => t('Date'),
          'description' => t('The Unix timestamp the message sent from the origin.'),
        ),
        'subject' => array(
          'name' => t('Subject'),
          'description' => t('The subject of the email message.'),
        ),
        'from_email' => array(
          'name' => t('From - Email'),
          'description' => t('The email address the message received from.'),
        ),
        'from_name' => array(
          'name' => t('From - Name'),
          'description' => t('The account name the message received from (not always available).'),
        ),
        'to_emails' => array(
          'name' => t('To emails'),
          'description' => t('A comma separated email address list the message sent to (not always available).'),
        ),
      );
      // Optionally add bodies related sources.
      if (!empty($fetcher->config['additional_options']['include_body'])) {
        $sources['body_plain'] = array(
          'name' => t('Body - Plain text'),
          'description' => t('The body of the email message in plain text format.'),
        );
        $sources['body_html'] = array(
          'name' => t('Body - HTML'),
          'description' => t('The body of the email message in HTML format.'),
        );
      }
      // Optionally add a flags related source.
      if (!empty($fetcher->config['additional_options']['include_flags'])) {
        $sources['flags'] = array(
          'name' => t('Flags'),
          'description' => t('A comma separated IMAP flag list the message marked with.'),
        );
      }
    }
    return $sources;
  }

  /**
   * Override parent::configDefaults().
   */
  public function configDefaults() {
    return array(
      'subject_replace' => array(
        'pattern' => '',
        'replacement' => '',
      ),
    );
  }

  /**
   * Override parent::configForm().
   */
  public function configForm(&$form_state) {
    $form = array('#tree' => TRUE);
    $form['subject_replace'] = array(
      '#type' => 'fieldset',
      '#title' => t('Subject replace'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
    );
    $form['subject_replace']['pattern'] = array(
      '#type' => 'textfield',
      '#title' => t('Pattern'),
      '#default_value' => $this->config['subject_replace']['pattern'],
      '#description' => t(
        "The regex pattern to search for. Do not include leading or closing pattern wrapper characters
        like slash (/)! Parenthesis can be used for references in the replacement field."),
    );
    $form['subject_replace']['replacement'] = array(
      '#type' => 'textfield',
      '#title' => t('Replacement'),
      '#default_value' => $this->config['subject_replace']['replacement'],
      '#description' => t(
        "The string to replace. May contain references of the form \$n or \${n}. Every
        such reference will be replaced by the text captured by the n'th parenthesized pattern in the
        pattern field. n can be from 0 to 99, and \$0 or \${0} refers to the text matched by the whole
        pattern."),
    );
    return $form;
  }

  /**
   * Override parent::configFormValidate().
   */
  public function configFormValidate(&$values) {
    if (!empty($values['subject_replace']['pattern']) && empty($values['subject_replace']['replacement'])) {
      form_set_error('subject_replace][replacement', t('Replacement cannot be empty if a pattern is given.'));
    }
    if (!empty($values['subject_replace']['replacement']) && empty($values['subject_replace']['pattern'])) {
      form_set_error('subject_replace][pattern', t('Pattern cannot be empty if a replacement is given.'));
    }
  }

}
